﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.Xml;
using System.Security.Cryptography.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Security.Cryptography.X509Certificates;


namespace Acme.Licensing
{
  /// <summary>
  /// License creation, signing and verification 
  /// </summary>
  internal class LicenseManager : LicenseManagerBase
  {
    public static void SaveLicense(LicenseInfo licenseInfo, string licenseFilePath)
    {
      XmlWriterSettings xmlWriterSettings = new XmlWriterSettings()
      {
        Encoding = new UTF8Encoding(false),
        ConformanceLevel = ConformanceLevel.Document,
        Indent = true
      };

      using (XmlWriter xmlWriter = XmlWriter.Create(licenseFilePath, xmlWriterSettings))
      {
        XmlDocument xmlDoc = new XmlDocument();
        XmlSerializer xmlSerializer = new XmlSerializer(typeof(LicenseInfo));
        xmlSerializer.Serialize(xmlWriter, licenseInfo);
      }

    }

    public static void SignLicense(string licenseFilePath, string pfxFilePath, string pfxFilePassword)
    {
      // Update IssueDate and Id
      LicenseInfo li;
      XmlSerializer xmlSerializer = new XmlSerializer(typeof(LicenseInfo));
      using (XmlReader xmlReader = XmlReader.Create(licenseFilePath))
      {
        li = (LicenseInfo)xmlSerializer.Deserialize(xmlReader);
      }

      li.LicenseId = Guid.NewGuid();
      li.IssueDate = DateTime.UtcNow.ToString(DateFormatString);

      // Save it back to the file
      SaveLicense(li, licenseFilePath);

      // Get certificate
      X509Certificate2 certificate = new X509Certificate2(pfxFilePath, pfxFilePassword);
      RSACryptoServiceProvider rsaCsp = (RSACryptoServiceProvider)certificate.PrivateKey;

      // Load license xml again
      XmlDocument xmlDoc = new XmlDocument();
      xmlDoc.Load(licenseFilePath);

      // Create signed xml object
      SignedXml signedXml = new SignedXml(xmlDoc);
      signedXml.SigningKey = rsaCsp;

      // Create a reference to be signed.
      Reference reference = new Reference();
      reference.Uri = "";

      XmlDsigEnvelopedSignatureTransform env = new XmlDsigEnvelopedSignatureTransform();
      reference.AddTransform(env);
      signedXml.AddReference(reference);

      // Compute signature
      signedXml.ComputeSignature();

      // Embed signature into the doc
      XmlElement xmlDigitalSignature = signedXml.GetXml();
      xmlDoc.DocumentElement.AppendChild(xmlDoc.ImportNode(xmlDigitalSignature, true));

      XmlWriterSettings xmlWriterSettings = new XmlWriterSettings()
      {
        Encoding = new UTF8Encoding(false),
        ConformanceLevel = ConformanceLevel.Document,
        Indent = true
      };
      using (XmlWriter w = XmlWriter.Create(licenseFilePath, xmlWriterSettings))
      {
        xmlDoc.WriteTo(w);
      }

      // Export public key so we can hardcode it in LicenseManagerBase.PublicKey
      string publicKey = certificate.PublicKey.Key.ToXmlString(false);
      using (StreamWriter w = new StreamWriter(PublicKeyFileName))
      {
        w.Write(publicKey);
      }
    }
  }
}
