﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Text;
using System.Reflection;

namespace Acme.Licensing
{
  class Program
  {
    static int Main(string[] args)
    {
      LicenseInfo li;
      try
      {
        li = LicenseManagerBase.GetLicenseInfo(Assembly.GetEntryAssembly().Location.Replace(Assembly.GetEntryAssembly().ManifestModule.Name, String.Empty) + LicenseManagerBase.LicenseFileName, LicenseManagerBase.PublicKey);
      }
      catch (Exception ex)
      {
        // No license file loaded, use trial settings
        li = LicenseManagerBase.GetTrialLicense();
        Console.ForegroundColor = ConsoleColor.Yellow;
        Console.WriteLine("License check failed: cannot obtain license info, assuming this is trial deployment. {0}", ex.Message);
        Console.ResetColor();
      }

      // Get the number of processor cores
      int coreCount = 0;
      foreach (var item in new System.Management.ManagementObjectSearcher("Select * from Win32_Processor").Get())
      {
        coreCount += int.Parse(item["NumberOfCores"].ToString());
      }

      if (coreCount > li.MaxCores)
      {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.WriteLine("License violation: actual number of processor cores ({0}) exceeds maximum licensed number ({1}). Consider upgrading your license.", coreCount, li.MaxCores);
      }
      else
      {
        // TODO: check license expiry date and other restrictions here if needed

        // Report success
        Console.ForegroundColor = ConsoleColor.Green;
        Console.WriteLine("License verification ok: actual number of processor cores ({0}) is less or equal than maximum licensed number ({1}).", coreCount, li.MaxCores);
      }
      Console.ResetColor();

      return 0;
    }
  }
}
